@@ -1,6 +1,7 @@
 # SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
 # SPDX-License-Identifier: MPL-2.0
 from typing import Any
+from typing import cast
 from typing import Optional
 
 from authlib.integrations.httpx_client import (
@@ -11,6 +12,7 @@ from authlib.oauth2 import OAuth2Client
 from httpx import USE_CLIENT_DEFAULT
 from httpx._types import AuthTypes
 from pydantic import AnyHttpUrl
+from pydantic import parse_obj_as
 
 
 class BaseAuthenticatedClient(OAuth2Client):
@@ -18,12 +20,11 @@ class BaseAuthenticatedClient(OAuth2Client):
         self,
         client_id: str,
         client_secret: str,
-        auth_server: AnyHttpUrl,
-        auth_realm: str,
+        token_endpoint: AnyHttpUrl,
         *args: Any,
-        grant_type: Optional[str] = "client_credentials",
-        token_endpoint_auth_method: Optional[str] = "client_secret_post",
-        **kwargs: Any
+        grant_type: str = "client_credentials",
+        token_endpoint_auth_method: str = "client_secret_post",
+        **kwargs: Any,
     ):
         """
         Base used to implement authenticated HTTPX clients. Does not work on its own.
@@ -31,38 +32,25 @@ class BaseAuthenticatedClient(OAuth2Client):
         Args:
             client_id: Client identifier used to obtain tokens.
             client_secret: Client secret used to obtain tokens.
-            auth_server: HTTP URL of the authentication server.
-            auth_realm: Keycloak realm used for authentication.
+            token_endpoint: OIDC token endpoint URL.
             *args: Other arguments, passed to Authlib's OAuth2Client.
             grant_type: OAuth2 grant type.
             token_endpoint_auth_method: RFC7591 client authentication method. Authlib
              supports 'client_secret_basic' (default), 'client_secret_post', and None.
             **kwargs: Other keyword arguments, passed to Authlib's OAuth2Client.
         """
-        self.auth_server = auth_server
-        self.auth_realm = auth_realm
+        self.token_endpoint = token_endpoint
 
         super().__init__(
             *args,
             client_id=client_id,
             client_secret=client_secret,
             grant_type=grant_type,
-            token_endpoint=self.token_endpoint,
+            token_endpoint=token_endpoint,
             token_endpoint_auth_method=token_endpoint_auth_method,
             **kwargs,
         )
 
-    @property
-    def token_endpoint(self) -> str:
-        """
-        Returns: Token endpoint based on given auth server and realm. Currently only
-         supports keycloak.
-        """
-        return "{server}/realms/{realm}/protocol/openid-connect/token".format(
-            server=self.auth_server,
-            realm=self.auth_realm,
-        )
-
     def should_fetch_token(
         self,
         url: str,
@@ -94,6 +82,24 @@ class BaseAuthenticatedClient(OAuth2Client):
         )
 
 
+def keycloak_token_endpoint(auth_server: AnyHttpUrl, auth_realm: str) -> AnyHttpUrl:
+    """Construct keycloak token endpoint based on the given auth server and realm.
+
+    Args:
+        auth_server: HTTP URL of the authentication server.
+        auth_realm: Keycloak realm used for authentication.
+
+    Returns: Token endpoint URL.
+    """
+    return cast(  # wtf is even going on. mypy is so dumb
+        AnyHttpUrl,
+        parse_obj_as(
+            AnyHttpUrl,
+            f"{auth_server}/realms/{auth_realm}/protocol/openid-connect/token",
+        ),
+    )
+
+
 class AuthenticatedHTTPXClient(BaseAuthenticatedClient, HTTPXOAuth2Client):
     """
     Synchronous HTTPX Client that automatically authenticates requests.
@@ -103,8 +109,10 @@ class AuthenticatedHTTPXClient(BaseAuthenticatedClient, HTTPXOAuth2Client):
         with AuthenticatedHTTPXClient(
             client_id="AzureDiamond",
             client_secret="hunter2",
-            auth_server=parse_obj_as(AnyHttpUrl, "http://keycloak.example.org/auth"),
-            auth_realm="mordor",
+            token_endpoint=keycloak_token_endpoint(
+                auth_server=parse_obj_as(AnyHttpUrl, "https://keycloak.example.org/auth"),
+                auth_realm="mordor",
+            ),
         ) as client:
             r = client.get("https://example.org")
 
@@ -116,7 +124,7 @@ class AuthenticatedHTTPXClient(BaseAuthenticatedClient, HTTPXOAuth2Client):
         url: str,
         withhold_token: bool = False,
         auth: AuthTypes = USE_CLIENT_DEFAULT,  # type: ignore[assignment]
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Any:
         """
         Decorate Authlib's OAuth2Client.request() to automatically fetch a token the
@@ -145,8 +153,10 @@ class AuthenticatedAsyncHTTPXClient(BaseAuthenticatedClient, AsyncHTTPXOAuth2Cli
         async with AuthenticatedAsyncHTTPXClient(
             client_id="AzureDiamond",
             client_secret="hunter2",
-            auth_server=parse_obj_as(AnyHttpUrl, "http://keycloak.example.org/auth"),
-            auth_realm="mordor",
+            token_endpoint=keycloak_token_endpoint(
+                auth_server=parse_obj_as(AnyHttpUrl, "https://keycloak.example.org/auth"),
+                auth_realm="mordor",
+            ),
         ) as client:
             r = await client.get("https://example.org")
 
@@ -158,7 +168,7 @@ class AuthenticatedAsyncHTTPXClient(BaseAuthenticatedClient, AsyncHTTPXOAuth2Cli
         url: str,
         withhold_token: bool = False,
         auth: AuthTypes = USE_CLIENT_DEFAULT,  # type: ignore[assignment]
-        **kwargs: Any
+        **kwargs: Any,
     ) -> Any:
         """
         Decorate Authlib's AsyncOAuth2Client.request() to automatically fetch a token
